<?php
/**
 * Visforms field class
 *
 * @author       Aicha Vack
 * @package      Joomla.Site
 * @subpackage   com_visforms
 * @link         https://www.vi-solutions.de
 * @license      GNU General Public License version 2 or later; see license.txt
 * @copyright    2012 vi-solutions
 * @since        Joomla 1.6
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\User\UserHelper;
use Joomla\CMS\Filter\InputFilter;
use Visolutions\Component\Visforms\Site\Model\VisformsModel as VisformsModelSite;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Uri\Uri;
use Visolutions\Component\Visforms\Administrator\Service\HTML\Visforms;

abstract class VisformsField
{
	protected $type;
	protected $field;
	protected $form;
	protected $value;
	protected $queryValue;
	protected $postValue;
	protected $input;
	protected $isEditOrSaveEditTask;

	public function __construct($field, $form) {
		$this->type = $field->typefield;
		$this->field = $field;
		$this->form = $form;
		$this->input = Factory::getApplication()->input;
		// make field names unique, allow use of one form more than once on one page
		if (!empty($this->form->context)) {
			$this->field->name = $this->form->context . $this->field->name;
		}
		$this->isEditOrSaveEditTask = ($this->form->displayState === VisformsModelSite::$displayStateIsNewEditData || $this->form->displayState === VisformsModelSite::$displayStateIsRedisplayEditData) ? true : false;
	}

	public static function getInstance($field, $form) {
		if (!(isset($field->typefield))) {
			return false;
		}

		$classname = get_called_class() . ucfirst($field->typefield);
		if (!class_exists($classname)) {
			// try to register it
			JLoader::register($classname, dirname(__FILE__) . '/field/' . $field->typefield . '.php');
			if (!class_exists($classname)) {
				// return a default class?
				return false;
			}
		}
		// delegate to the appropriate subclass
		return new $classname($field, $form);
	}

	public function getField() {
		$this->setField();
		return $this->field;
	}

	protected function extractDefaultValueParams() {
		$this->field->defaultvalue = \VisformsHelper::registryArrayFromString($this->field->defaultvalue);
		foreach ($this->field->defaultvalue as $name => $value) {
			// make names shorter and set all default values as properties of field object
			$prefix = 'f_' . $this->field->typefield . '_';
			if (str_contains($name, $prefix)) {
				$key = str_replace($prefix, "", $name);
				$this->field->$key = $value;
			}
		}

        // delete defaultvalue array
        unset($this->field->defaultvalue);
    }

	protected function extractGridSizesParams() {

		$this->field->gridSizes = \VisformsHelper::registryArrayFromString($this->field->gridSizes);
		foreach ($this->field->gridSizes as $key => $value) {
			$this->field->$key = $value;
		}
		unset($this->field->gridSizes);
	}

	protected function mendBooleanAttribs() {
		$attribs = array('required' => true, 'readonly' => 'readonly', 'checked' => 'checked');
		foreach ($attribs as $attrib => $value) {
			$attribname = 'attribute_' . $attrib;
			// Do not remove. Older fields may have set the attribute with empty walue!
			if (isset($this->field->$attribname) && ($this->field->$attribname == $attrib || $this->field->$attribname == '1' || $this->field->$attribname == true)) {
				$this->field->$attribname = $attrib;
			}
			else {
				unset($this->field->$attribname);
			}
		}
	}

	protected function extractRestrictions() {
		$this->field->restrictions = \VisformsHelper::registryArrayFromString($this->field->restrictions);
	}

	protected function setIsConditional() {
		foreach ($this->field as $name => $avalue) {
			if (str_contains($name, 'showWhen')) {
				// as there can be more than one restrict, restricts are stored in an array
				if (is_array($avalue) && (count($avalue) > 0)) {
					foreach ($avalue as $value) {
						// if we have at least on restict with a field there is a condition set
						if (preg_match('/^field/', $value) === 1) {
							$this->field->isConditional = true;
							return;
						}
					}
				}
			}

        }
        $this->field->isConditional = false;
    }

	protected function setIsDisplayChanger() {
		if (isset($this->field->restrictions) && (is_array($this->field->restrictions))) {
			//loop through restrictions and check that there is at least one usedAsShowWhen restriction
			if (array_key_exists('usedAsShowWhen', $this->field->restrictions)) {
				$this->field->isDisplayChanger = true;
			}
		}
	}

	protected function setErrorMessageInForm($error) {
	    if ($error === '') {
	        return;
        }
		if (!(isset($this->form->errors))) {
			$this->form->errors = array();
		}
		if (is_array($this->form->errors)) {
			array_push($this->form->errors, $error);
		}
	}

	protected function escapeCustomRegex() {
		if (!(isset($this->field->customvalidation))) {
			return;
		}
		$clean1 = str_replace("\/", "/", $this->field->customvalidation);
		$clean2 = str_replace("/", "\/", str_replace("\/", "/", $clean1));
		$this->field->customvalidation = $clean2;
	}

	public function setRecordId($cid) {
		$this->field->recordId = $cid;
	}

	protected function addFormStep() {
		if ((isset($this->form->steps)) && (is_numeric($this->form->steps))) {
			$this->form->steps++;
		} 
		else {
			$this->form->steps = 1;
		}
        if (empty($this->form->canHideSummaryOnMultiPageForms) && ($this->form->steps > 1)) {
            $this->form->displaysummarypage = true;
        }
    }

	protected function hasSearchableSelect() {
		if (!empty($this->field->is_searchable)) {
			$this->form->hasSearchableSelect = true;
		}
	}

	protected function setFieldsetCounter() {
		$this->field->fieldsetcounter = $this->form->steps;
	}

	// create empty array as field parameter
	// can be used in business class of the field
	protected function setCustomJs() {
		$this->field->customJs = array();
	}

    abstract protected function setRedirectParam();

	// Method to convert post values into a string that can be stored in db and attach it as property to the field object
    abstract protected function setDbValue();

	// Set the default value of the field which is displayed in the form according field definition, query params, user inputs
	abstract protected function setFieldDefaultValue();

	protected function setQueryValue() {
		if ($this->form->displayState === VisformsModelSite::$displayStateIsNew) {
			$app = Factory::getApplication();
			$task = $app->input->getCmd('task', '');
			if (($task !== 'editdata') && ($task !== 'saveedit')) {
				// using $this->input->get->get makes sure that the joomla! security functions are performed on the user inputs!
				// plugin form view sets get values as well
				$queryValue = $this->input->get->get($this->field->name, null, 'STRING');
				if (!is_null($queryValue)) {
					$urlparams = $app->getUserState('com_visforms.urlparams.' . $this->form->context);
					if (empty($urlparams)) {
						$urlparams = array();
					}
					$urlparams[$this->field->name] = (!empty($this->field->decodeqpvalue)) ? HTMLHelper::_('visforms.base64_url_decode', $queryValue) : $queryValue;
					$app->setUserState('com_visforms.urlparams.' . $this->form->context, $urlparams);
				}
			}
		}
	}

    protected function setEnterKeyAction() {
    	if (!empty($this->form->preventsubmitonenter)) {
    		$this->field->disableEnterKey = true;
	    }
    }

	protected function mendInvalidUncheckedValue() {
		if (!empty(\VisformsAEF::checkAEF(VisformsAEF::$subscription))) {
			if (!isset($this->field->unchecked_value) || $this->field->unchecked_value === "") {
				$this->field->unchecked_value = 0;
			}
			$this->field->unchecked_value = trim(str_replace(",", ".", $this->field->unchecked_value));
		}
	}

	protected function setShowRequiredAsterix() {
		if (!empty($this->form->requiredasterix)) {
			$this->field->showRequiredAsterix = true;
		}
	}

	protected function getParameterFieldNameForEditValue() {
		return "F" . $this->field->id;
	}

	protected function setEditValue() {
		if ($this->isEditOrSaveEditTask) {
			$this->field->editValue = "";
			$data = $this->form->data;
			$dataFieldName = $this->getParameterFieldNameForEditValue();
			if (isset($data->$dataFieldName)) {
				$filter = InputFilter::getInstance();
				$this->field->editValue = $filter->clean($data->$dataFieldName, 'STRING');
			}
		}
	}
    // override a default value set in attribute_value if fillwith is not set to '0'
    protected function fillWith() {
        $field = $this->field;
        // no custom default value selection.
        // don't change 'attribute_value'
        if ((!isset($field->fillwith) || $field->fillwith === "0")) {
            return;
        }
        // process fillwith sql first
        if ($field->fillwith === 'sql') {
            try {
                $sqlHelper = new \VisformsSql($field->defaultvalue_sql);
                $this->field->attribute_value = $sqlHelper->getItemsFromSQL('loadResult');
            }
            catch (\Exception $e) {
                $this->field->attribute_value = '';
            }
            return;
        }
        // this default value does not depend on the user being logged in
        if ($field->fillwith == "1") {
            $this->field->attribute_value = uniqid($this->field->attribute_value, true);
            return;
        }
        if ($field->fillwith == 'url') {
            $this->field->attribute_value = Uri::getInstance()->toString();
            return;
        }
        // process fillwith user data
        // if we edit stored user inputs we want to use the user profile of the user who submitted the form as default fill with values
        if (($this->form->displayState === VisformsModelSite::$displayStateIsNewEditData) || ($this->form->displayState === VisformsModelSite::$displayStateIsRedisplayEditData)) {
            $data = $this->form->data;
            $userId = $data->created_by;
            if (!empty($userId)) {
                $user = Factory::getUser($userId);
            }
        }
        // use user profile of logged-in user
        else {
            $user = Factory::getApplication()->getIdentity();
            $userId = $user->get('id');
        }
        if ($userId != 0) {
            if ($field->fillwith == "2") {
                $this->field->attribute_value = $user->get('name');
                return;
            }
            if ($field->fillwith == "3") {
                $this->field->attribute_value = $user->get('username');
                return;
            }
            if ($field->fillwith == 'usermail') {
                $this->field->attribute_value = $user->get('email');
                return;
            }
            preg_match('/^CF(\d+)$/', $field->fillwith, $matches);
            if ($matches && !empty($matches[1])) {
                $customfieldid = $matches[1];
                $value = HTMLHelper::_('visforms.getCustomUserFieldValue', $customfieldid, $user);
                // in error case, value === false
                if ($value !== false) {
                    $this->field->attribute_value = $value ;
                    return;
                }
            }
            $userProfile = UserHelper::getProfile($userId);
            if ((!(empty($userProfile->profile))) && (is_array($userProfile->profile))) {
                if (!(empty($userProfile->profile[$field->fillwith]))) {
                    $this->field->attribute_value = $userProfile->profile[$field->fillwith];
                    return;
                }
            }
        }
        // no value found, set attribute_value to ''
        $this->field->attribute_value = '';
    }

	protected function replaceLabelWithCustomText() {
        if (!empty(\VisformsAEF::checkAEF(VisformsAEF::$subscription))) {
            if (!empty($this->field->customtext) && isset($this->field->customtextposition) && $this->field->customtextposition == "3") {
                // remove HTML block elements from custom text using Joomla input filter (list from https://www.w3schools.com/html/html_blocks.asp)
                $customBlocked =  array('p', 'div', 'pre', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6',
                    'address', 'article', 'aside', 'blockquote',
                    'canvas', 'dd', 'dl', 'dt', 'fieldset', 'figcaption', 'figure', 'footer', 'form', 'header', 'hr', 'li', 'main', 'nav', 'noscript', 'ol', 'pre', 'section', 'table', 'tfoot', 'ul', 'video');
                $defaultBlocked = InputFilter::getInstance()->blockedTags; // Joomla default block list
                $blockList = array_merge($customBlocked, $defaultBlocked);
                $label = InputFilter::getInstance()->emailToPunycode($this->field->customtext);
                $filter = InputFilter::getInstance(
                   $blockList, // tag block list
                    [], // attribute block list
                    InputFilter::ONLY_BLOCK_DEFINED_TAGS,
                    InputFilter::ONLY_BLOCK_DEFINED_ATTRIBUTES
                );
                $label = trim($filter->clean($label, 'html'));
                // replace label, if we still have any text
                if (!empty($label)) {
                    $this->field->label = $label;
                }
            }
        }
    }

    protected function prepareFieldForNoEditSqlProcess() {
        // new option which might not be set in older fields
        // Default is 'process' with a value of 1
        // so only if option is set and has the value 0 no process of sql in edit view
        if ($this->isEditOrSaveEditTask && ((isset($this->field->sql_process_in_edit_view) && empty($this->field->sql_process_in_edit_view)))) {
            $this->field->noEditSqlProcess = true;
        }
    }

    protected function setCustomPhpErrorMsg() {
	    if (empty($this->field->custom_php_error)) {
            $this->field->custom_php_error = '';
        }
    }

    protected function unsetDeselectedEditViewValidations() {
        // check for each validation option if it was deselected for edit view
        // new options which might not be set in older fields
        // Default is 'validate in edit view' with a value of 1
        // so only if option is set and has the value 0 validation should not be performed in edit view
        if ($this->isEditOrSaveEditTask ) {
            foreach ($this->field as $name => $value) {
                if (!is_array($value)) {
                    if ($value) {
                        // field option starts with validate_
                        $ruleName = '';
                        $ruleNevvName='';
                        if (str_contains($name, 'validate')) {
                            $ruleName = str_replace('validate_', "", strtolower($name));
                            $ruleNevvName = $ruleName . '_nevv';
                            if (!empty($this->field->$ruleNevvName)) {
                                unset($this->field->$name);
                                continue;
                            }
                        }
                        // field option with a custom name
                        if (in_array($name, Visforms::customValidationNames)) {
                            $ruleNevvName = $name . '_nevv';
                            if (!empty($this->field->$ruleNevvName)) {
                                unset($this->field->$name);
                                continue;
                            }
                        }
                        // field option starts with attribute
                        if (str_contains($name, 'attribute_')) {
                            $ruleName = str_replace('attribute_', "", $name);
                            $ruleNevvName = $ruleName . '_nevv';
                            if (in_array($ruleName, Visforms::htmlValidationAttribs)) {
                                if (!empty($this->field->$ruleNevvName)) {
                                    unset($this->field->$name);
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}